@@ -266,10 +266,10 @@ inline auto toString(Arg&& arg) -> typename std::enable_if<std::is_same_v<String
 
  /// Returns the function argument of the type matching the template argument. \sa fzS
 template<typename Char>
-Char const* choose_string(char const* c, wchar_t const* w);
+constexpr Char const* choose_string(char const* c, wchar_t const* w);
 
-template<> inline char const* choose_string(char const* c, wchar_t const*) { return c; }
-template<> inline wchar_t const* choose_string(char const*, wchar_t const* w) { return w; }
+template<> constexpr inline char const* choose_string(char const* c, wchar_t const*) { return c; }
+template<> constexpr inline wchar_t const* choose_string(char const*, wchar_t const* w) { return w; }
 
 #if !defined(fzS) || defined(DOXYGEN)
 /** \brief Macro to get const pointer to a string of the corresponding type
@@ -761,6 +761,14 @@ bool FZ_PUBLIC_SYMBOL utf16be_to_utf8_append(std::string & result, std::string_v
 /// Just as utf16be_to_utf8_append but for little-endian UTF-16.
 bool FZ_PUBLIC_SYMBOL utf16le_to_utf8_append(std::string & result, std::string_view data, uint32_t & state);
 
+inline native_string to_native_from_utf8(std::string_view s) {
+#ifdef FZ_WINDOWS
+	return to_wstring_from_utf8(s);
+#else
+	return to_string(to_wstring_from_utf8(s));
+#endif
+}
+
 }
 
 #endif
