@@ -23,6 +23,58 @@ struct certificate_verification_event_type;
  */
 typedef simple_event<certificate_verification_event_type, tls_layer*, tls_session_info> certificate_verification_event;
 
+enum class tls_ver
+{
+	v1_0,
+	v1_1,
+	v1_2,
+	v1_3
+};
+
+enum class tls_server_flags : unsigned int
+{
+	none = 0,
+
+	/// In TLS 1.3, do not automatically send PSKs after finishing handshake. Ignored if not TLS 1.3
+	no_auto_ticket = 0x1,
+
+	/// Used in unit tests. Don't use in production.
+	debug_no_tickets = 0x2
+};
+
+inline bool operator&(tls_server_flags lhs, tls_server_flags rhs) {
+	return (static_cast<std::underlying_type_t<tls_server_flags>>(lhs) & static_cast<std::underlying_type_t<tls_server_flags>>(rhs)) != 0;
+}
+inline tls_server_flags operator|(tls_server_flags lhs, tls_server_flags rhs) {
+	return static_cast<tls_server_flags>(static_cast<std::underlying_type_t<tls_server_flags>>(lhs) | static_cast<std::underlying_type_t<tls_server_flags>>(rhs));
+}
+inline tls_server_flags& operator|=(tls_server_flags & lhs, tls_server_flags rhs) {
+	lhs = lhs | rhs;
+	return lhs;
+}
+
+
+enum class tls_client_flags : unsigned int
+{
+	none = 0,
+
+	/// Used in unit tests. Don't use in production.
+	debug_no_tickets = 0x1
+};
+
+inline bool operator&(tls_client_flags lhs, tls_client_flags rhs) {
+	return (static_cast<std::underlying_type_t<tls_client_flags>>(lhs) & static_cast<std::underlying_type_t<tls_client_flags>>(rhs)) != 0;
+}
+inline tls_client_flags operator|(tls_client_flags lhs, tls_client_flags rhs) {
+	return static_cast<tls_client_flags>(static_cast<std::underlying_type_t<tls_client_flags>>(lhs) | static_cast<std::underlying_type_t<tls_client_flags>>(rhs));
+}
+inline tls_client_flags& operator|=(tls_client_flags & lhs, tls_client_flags rhs) {
+	lhs = lhs | rhs;
+	return lhs;
+}
+
+
+
 /**
  * \brief A Transport Layer Security (TLS) layer
  *
@@ -52,7 +104,7 @@ public:
 	 * must match the passed \c required_certificate, either in DER or PEM,
 	 * or the handshake will fail.
 	 */
-	bool client_handshake(std::vector<uint8_t> const& required_certificate, std::vector<uint8_t> const& session_to_resume = std::vector<uint8_t>(), native_string const& session_hostname = native_string());
+	bool client_handshake(std::vector<uint8_t> const& required_certificate, std::vector<uint8_t> const& session_to_resume = std::vector<uint8_t>(), native_string const& session_hostname = native_string(), tls_client_flags flags = {});
 
 	/**
 	 * \brief Starts shaking hands for a new TLS session as client.
@@ -70,7 +122,7 @@ public:
 	 * The handler is called even for certificates not trusted by the system trust store, allowing
 	 * the following impairments: Unknown issuer, wrong hostname and certificates used outside their validity time.
 	 */
-	bool client_handshake(event_handler *const verification_handler, std::vector<uint8_t> const& session_to_resume = std::vector<uint8_t>(), native_string const& session_hostname = native_string());
+	bool client_handshake(event_handler *const verification_handler, std::vector<uint8_t> const& session_to_resume = std::vector<uint8_t>(), native_string const& session_hostname = native_string(), tls_client_flags flags = {});
 
 	/**
 	 * \brief Starts shaking hand for a new TLS session as server.
@@ -87,7 +139,7 @@ public:
 	 *
 	 * The preamble is sent out after setting up all the parameters, but before the first handshake message
 	 */
-	bool server_handshake(std::vector<uint8_t> const& session_to_resume = {}, std::string_view const& preamble = {});
+	bool server_handshake(std::vector<uint8_t> const& session_to_resume = {}, std::string_view const& preamble = {}, tls_server_flags flags = {});
 
 	/// Gets session parameters for resumption
 	std::vector<uint8_t> get_session_parameters() const;
@@ -133,11 +185,19 @@ public:
 	 *
 	 * If the pem flag is set, the input is assumed to be in PEM, otherwise DER.
 	 */
-	bool set_certificate(std::string const& key, std::string const& certs, native_string const& password, bool pem = true);
+	bool set_certificate(std::string_view const& key, std::string_view const& certs, native_string const& password, bool pem = true);
 
 	/// Returns the version of the loaded GnuTLS library, may be different than the version used at compile-time.
 	static std::string get_gnutls_version();
 
+	/// Type of certificate to create.
+	enum class cert_type {
+		any, /// Except ca. Includes, but is not limited to, client and server.
+		client,
+		server,
+		ca
+	};
+
 	/** \brief Creates a new private key and a self-signed certificate.
 	 *
 	 * The distinguished name must be a RFC4514-compliant string.
@@ -146,15 +206,45 @@ public:
 	 *
 	 * The output pair is in PEM, first element is the key and the second the certificate.
 	 */
-	static std::pair<std::string, std::string> generate_selfsigned_certificate(native_string const& password, std::string const& distinguished_name, std::vector<std::string> const& hostnames);
+	static std::pair<std::string, std::string> generate_selfsigned_certificate(native_string const& password, std::string const& distinguished_name, std::vector<std::string> const& hostnames, cert_type type = cert_type::any, bool ecsda = true);
+
+	/// Creates CA certificate.
+	static std::pair<std::string, std::string> generate_ca_certificate(native_string const& password, std::string const& distinguished_name, duration const& lifetime = {}, bool ecdsa = true);
+
+	/// Creates a CSR
+	static std::pair<std::string, std::string> generate_csr(native_string const& password, std::string const& distinguished_name, std::vector<std::string> const& hostnames, bool csr_as_pem = true, cert_type type = cert_type::any);
+
+	/**
+	 * \brief Creates a certificate from a CSR.
+	 *
+	 * If DN and hostnames are not given, the DN/hostnames from the CSR are taken.
+	 */
+	static std::string generate_cert_from_csr(std::pair<std::string, std::string> const& issuer, native_string const& password, std::string const& csr, std::string const& distinguished_name = {}, std::vector<std::string> const& hostnames = {}, duration const& lifetime = {}, cert_type type = cert_type::any);
 
 	/** \brief Negotiate application protocol
 	 *
 	 * If the peer makes use of ALPN, the handshake fails if no matching protocol is found.
 	 * If the peer does not use/support ALPN, the handshake continues and no protocol is negotiated.
+	 * 
+	 * Needs to be called prior to handshaking.
+	 *
+	 * Which protocol is eventually chosen is always a server-side decsion.
+	 * Iff running as server, server_priority controls the ALPN should be chosen
+	 * based on the server's or the client's order in the list of ALPNs supported
+	 * by both. server_priority is ignored if unning as client.
 	 */
 	bool set_alpn(std::string_view const& alpn);
-	bool set_alpn(std::vector<std::string> const& alpns);
+	bool set_alpn(std::vector<std::string> const& alpns, bool server_priority = false);
+
+	/** \brief Sets minimum allowed TLS version
+	 */
+	void set_min_tls_ver(tls_ver ver);
+
+	/** \brief Sets maximum allowed TLS versions
+	 *
+	 * Don't set a max version in production, it is for testing things.
+	 */
+	void set_max_tls_ver(tls_ver ver);
 
 	/// After a successful handshake, returns which protocol, if any, has been negotiated
 	std::string get_alpn() const;
@@ -164,6 +254,32 @@ public:
 
 	bool is_server() const;
 
+	/** \brief If running as server with TLS1.3, send out a new session ticket before the next data payload.
+	 *
+	 * Returns 0 on success, socket error otheerwise. Never returns EAGAIN.
+	 *
+	 * It's a NOOP if TLS version != 1.3
+	 *
+	 * Returns EINVAL if not server.
+	 */
+	int new_session_ticket();
+
+	/** \brief Sets a callback to control whether unexpected eof is seen as error
+	 *
+	 * With TLS, an EOF on the socket prior to receiving a closure alert normally is an error.
+	 * In many cases this is harmless though, e.g. if the connection is idle.
+	 *
+	 * If this callback is set, premature termination is no longer seen as error if the
+	 * callback returns false.
+	 *
+	 * Callback must not call any tls_layer function.
+	 *
+	 * Only controls the layer's own logging. Functions such as read will still return
+	 * ECONNABORTED.
+	 */
+	void set_unexpected_eof_cb(std::function<bool()> const& cb);
+	void set_unexpected_eof_cb(std::function<bool()> && cb);
+
 	virtual socket_state get_state() const override;
 
 	virtual int connect(native_string const& host, unsigned int port, address_type family = address_type::unknown) override;
